@@ -59,6 +59,52 @@
 
 #define MAX_VERSION_LENGTH 64
 
+/* NM_IPAC_F_CHANT_* mask for NM_IPAC_EIE_CHAN_TYPES (common) */
+#define NM_IPAC_MASK_CHANT_COMMON \
+	(NM_IPAC_F_CHANT_TCHF		| \
+	 NM_IPAC_F_CHANT_TCHH		| \
+	 NM_IPAC_F_CHANT_SDCCH8		| \
+	 NM_IPAC_F_CHANT_BCCH		| \
+	 NM_IPAC_F_CHANT_BCCH_SDCCH4)
+/* NM_IPAC_F_CHANM_SPEECH_* mask for NM_IPAC_EIE_CHAN_MODES */
+#define NM_IPAC_MASK_CHANM_SPEECH \
+	(NM_IPAC_F_CHANM_SPEECH_FS	| \
+	 NM_IPAC_F_CHANM_SPEECH_EFS	| \
+	 NM_IPAC_F_CHANM_SPEECH_AFS	| \
+	 NM_IPAC_F_CHANM_SPEECH_HS	| \
+	 NM_IPAC_F_CHANM_SPEECH_AHS)
+/* NM_IPAC_F_CHANM_CSD_NT_* mask for NM_IPAC_EIE_CHAN_MODES */
+#define NM_IPAC_MASK_CHANM_CSD_NT \
+	(NM_IPAC_F_CHANM_CSD_NT_4k8	| \
+	 NM_IPAC_F_CHANM_CSD_NT_9k6	| \
+	 NM_IPAC_F_CHANM_CSD_NT_14k4)
+/* NM_IPAC_F_CHANM_CSD_T_* mask for NM_IPAC_EIE_CHAN_MODES */
+#define NM_IPAC_MASK_CHANM_CSD_T \
+	(NM_IPAC_F_CHANM_CSD_T_1200_75	| \
+	 NM_IPAC_F_CHANM_CSD_T_600	| \
+	 NM_IPAC_F_CHANM_CSD_T_1k2	| \
+	 NM_IPAC_F_CHANM_CSD_T_2k4	| \
+	 NM_IPAC_F_CHANM_CSD_T_4k8	| \
+	 NM_IPAC_F_CHANM_CSD_T_9k6	| \
+	 NM_IPAC_F_CHANM_CSD_T_14k4)
+/* NM_IPAC_F_GPRS_CODING_CS[1-4] mask for NM_IPAC_EIE_GPRS_CODING */
+#define NM_IPAC_MASK_GPRS_CODING_CS \
+	(NM_IPAC_F_GPRS_CODING_CS1	| \
+	 NM_IPAC_F_GPRS_CODING_CS2	| \
+	 NM_IPAC_F_GPRS_CODING_CS3	| \
+	 NM_IPAC_F_GPRS_CODING_CS4)
+/* NM_IPAC_F_GPRS_CODING_MCS[1-9] mask for NM_IPAC_EIE_GPRS_CODING */
+#define NM_IPAC_MASK_GPRS_CODING_MCS \
+	(NM_IPAC_F_GPRS_CODING_MCS1	| \
+	 NM_IPAC_F_GPRS_CODING_MCS2	| \
+	 NM_IPAC_F_GPRS_CODING_MCS3	| \
+	 NM_IPAC_F_GPRS_CODING_MCS4	| \
+	 NM_IPAC_F_GPRS_CODING_MCS5	| \
+	 NM_IPAC_F_GPRS_CODING_MCS6	| \
+	 NM_IPAC_F_GPRS_CODING_MCS7	| \
+	 NM_IPAC_F_GPRS_CODING_MCS8	| \
+	 NM_IPAC_F_GPRS_CODING_MCS9)
+
 enum gsm_bts_trx_ts_flags {
 	TS_F_PDCH_ACTIVE =		0x1000,
 	TS_F_PDCH_ACT_PENDING =		0x2000,
