@@ -4,6 +4,7 @@
 #include <stdint.h>
 #include <netinet/in.h>
 
+#include <osmocom/core/bits.h>
 #include <osmocom/core/timer.h>
 #include <osmocom/core/linuxlist.h>
 #include <osmocom/core/logging.h>
@@ -88,6 +89,12 @@ enum lchan_csd_mode {
 	_LCHAN_CSD_M_NUM,
 };
 
+extern const struct value_string lchan_csd_mode_descs[];
+static inline const char *lchan_csd_mode_desc(enum lchan_csd_mode mode)
+{
+	return get_value_string(lchan_csd_mode_descs, mode);
+}
+
 /* State of the SAPIs in the lchan */
 enum lchan_sapi_state {
 	LCHAN_SAPI_S_NONE,
@@ -285,13 +292,21 @@ struct gsm_lchan {
 			bool dl_sid_transmitted;
 			/* The current frame in the DL is taken up by FACCH */
 			bool dl_facch_stealing;
+			/* UL SID filter to catch DTXu half-blocks,
+			 * see tch_ul_fr_hr_efr() function. */
+			bool ul_sid_filter;
 		} dtx_fr_hr_efr;
 		uint8_t last_cmr;
 		uint32_t last_fn;
 		struct {
-			/* buffers to re-combine RLP frame from multiple Um blocks */
+			/* RLP GSMTAP mechanism */
 			uint8_t rlp_buf_ul[576/8]; /* maximum size of RLP frame */
 			uint8_t rlp_buf_dl[576/8]; /* maximum size of RLP frame */
+			/* alignment of RLP frames in DL for NT modes */
+			ubit_t rlpdl_data_bits[60 * 7];
+			uint16_t rlpdl_align_bits;
+			uint8_t rlpdl_fill_level;
+			ubit_t tchf48_nt_2ndhalf[120];
 		} csd;
 	} tch;
 
