@@ -14,7 +14,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -49,6 +49,8 @@
 #include "oc2gbts.h"
 #include "utils.h"
 
+static void dump_lch_par(int logl, GsmL1_LogChParam_t *lch_par, GsmL1_Sapi_t sapi);
+
 static int mph_info_chan_confirm(struct gsm_lchan *lchan,
 			enum osmo_mph_info_type type, uint8_t cause)
 {
@@ -471,7 +473,7 @@ uint32_t trx_get_hlayer1(const struct gsm_bts_trx *trx)
 {
 	const struct oc2gl1_hdl *fl1h = trx_oc2gl1_hdl(trx);
 
-	return fl1h->hLayer1;
+	return (uint32_t)fl1h->hLayer1;
 }
 
 static int trx_close_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
@@ -584,8 +586,7 @@ GsmL1_Sapi_t lchan_to_GsmL1_Sapi_t(const struct gsm_lchan *lchan)
 	case GSM_LCHAN_TCH_H:
 		return GsmL1_Sapi_TchH;
 	default:
-		LOGP(DL1C, LOGL_NOTICE, "%s cannot determine L1 SAPI\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_NOTICE, "cannot determine L1 SAPI\n");
 		break;
 	}
 	return GsmL1_Sapi_Idle;
@@ -796,12 +797,8 @@ static void sapi_queue_dispatch(struct gsm_lchan *lchan, int status)
 	talloc_free(cmd);
 
 	if (end || llist_empty(&lchan->sapi_cmds)) {
-		LOGP(DL1C, LOGL_DEBUG,
-			"%s End of SAPI cmd queue encountered.%s\n",
-			gsm_lchan_name(lchan),
-			llist_empty(&lchan->sapi_cmds)
-				? " Queue is now empty."
-				: " More pending.");
+		LOGPLCHAN(lchan, DL1C, LOGL_DEBUG, "End of SAPI cmd queue encountered.%s\n",
+			  llist_empty(&lchan->sapi_cmds) ? " Queue is now empty." : " More pending.");
 		return;
 	}
 
@@ -841,9 +838,8 @@ static int lchan_act_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		goto err;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-ACTIVATE.conf (%s ",
-		gsm_lchan_name(lchan),
-		get_value_string(oc2gbts_l1sapi_names, ic->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-ACTIVATE.conf (%s ",
+		  get_value_string(oc2gbts_l1sapi_names, ic->sapi));
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(oc2gbts_dir_names, ic->dir));
 
@@ -864,19 +860,15 @@ static int lchan_act_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		lchan->sapis_ul[ic->sapi] = status;
 
 	if (llist_empty(&lchan->sapi_cmds)) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Got activation confirmation with empty queue\n",
-				gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Got activation confirmation with empty queue\n");
 		goto err;
 	}
 
 	cmd = llist_entry(lchan->sapi_cmds.next, struct sapi_cmd, entry);
 	if (cmd->sapi != ic->sapi || cmd->dir != ic->dir ||
 			cmd->type != SAPI_CMD_ACTIVATE) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Confirmation mismatch (%d, %d) (%d, %d)\n",
-				gsm_lchan_name(lchan), cmd->sapi, cmd->dir,
-				ic->sapi, ic->dir);
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Confirmation mismatch (%d, %d) (%d, %d)\n",
+			  cmd->sapi, cmd->dir, ic->sapi, ic->dir);
 		goto err;
 	}
 
@@ -957,8 +949,7 @@ static int lchan2lch_par(GsmL1_LogChParam_t *lch_par, struct gsm_lchan *lchan)
 			(struct gsm48_multi_rate_conf *) amr_mrc->gsm48_ie;
 	int j;
 
-	LOGP(DL1C, LOGL_INFO, "%s: %s tch_mode=0x%02x\n",
-		gsm_lchan_name(lchan), __FUNCTION__, lchan->tch_mode);
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "%s tch_mode=0x%02x\n", __func__, lchan->tch_mode);
 
 	switch (lchan->tch_mode) {
 	case GSM48_CMODE_SIGN:
@@ -986,7 +977,9 @@ static int lchan2lch_par(GsmL1_LogChParam_t *lch_par, struct gsm_lchan *lchan)
 	case GSM48_CMODE_SPEECH_AMR:
 		lch_par->tch.tchPlType = GsmL1_TchPlType_Amr;
 		set_payload_format(lch_par);
-		lch_par->tch.amrCmiPhase = GsmL1_AmrCmiPhase_Odd; /* FIXME? */
+		/* At call set-up, after every successful handover and after a channel mode modify, the
+		 * default phase (odd) shall be used in downlink direction. */
+		 lch_par->tch.amrCmiPhase = GsmL1_AmrCmiPhase_Odd;
 		lch_par->tch.amrInitCodecMode = amr_get_initial_mode(lchan);
 
 		/* initialize to clean state */
@@ -1109,9 +1102,9 @@ static int mph_send_activate_req(struct gsm_lchan *lchan, struct sapi_cmd *cmd)
 		break;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-ACTIVATE.req (hL2=0x%08x, %s ",
-		gsm_lchan_name(lchan), (uint32_t)act_req->hLayer2,
-		get_value_string(oc2gbts_l1sapi_names, act_req->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-ACTIVATE.req (hL2=0x%08x, %s ",
+		  (uint32_t)act_req->hLayer2, get_value_string(oc2gbts_l1sapi_names, act_req->sapi));
+	dump_lch_par(LOGL_INFO, lch_par, act_req->sapi);
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(oc2gbts_dir_names, act_req->dir));
 
@@ -1135,9 +1128,7 @@ static int sapi_activate_cb(struct gsm_lchan *lchan, int status)
 
 	/* FIXME: Error handling */
 	if (status != GsmL1_Status_Success) {
-		LOGP(DL1C, LOGL_ERROR,
-			"%s act failed mark broken due status: %d\n",
-			gsm_lchan_name(lchan), status);
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "act failed mark broken due status: %d\n", status);
 		lchan_set_state(lchan, LCHAN_S_BROKEN);
 		sapi_clear_queue(&lchan->sapi_cmds);
 		mph_info_chan_confirm(lchan, PRIM_INFO_ACTIVATE, RSL_ERR_PROCESSOR_OVERLOAD);
@@ -1184,13 +1175,11 @@ int lchan_activate(struct gsm_lchan *lchan)
 	lchan_set_state(lchan, LCHAN_S_ACT_REQ);
 
 	if (!llist_empty(&lchan->sapi_cmds))
-		LOGP(DL1C, LOGL_ERROR,
-			"%s Trying to activate lchan, but commands in queue\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Trying to activate lchan, but commands in queue\n");
 
 	/* For handover, always start the main channel immediately. lchan->want_dl_sacch_active indicates whether dl
 	 * SACCH should be activated. Also, for HO, start the RACH SAPI. */
-	if (lchan->ho.active == HANDOVER_ENABLED)
+	if (lchan->ho.active == HANDOVER_ENABLED || rsl_chan_rt_is_asci(lchan->rsl_chan_rt))
 		enqueue_sapi_act_cmd(lchan, GsmL1_Sapi_Rach, GsmL1_Dir_RxUplink);
 
 	for (i = 0; i < s4l->num_sapis; i++) {
@@ -1343,9 +1332,8 @@ static int chmod_modif_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		goto err;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-CONFIG.conf (%s) ",
-		gsm_lchan_name(lchan),
-		get_value_string(oc2gbts_l1cfgt_names, cc->cfgParamId));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-CONFIG.conf (%s) ",
+		  get_value_string(oc2gbts_l1cfgt_names, cc->cfgParamId));
 
 	switch (cc->cfgParamId) {
 	case GsmL1_ConfigParamId_SetLogChParams:
@@ -1377,9 +1365,7 @@ static int chmod_modif_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 			break;
 		}
 		if (llist_empty(&lchan->sapi_cmds)) {
-			LOGP(DL1C, LOGL_ERROR,
-				"%s Got ciphering conf with empty queue\n",
-				gsm_lchan_name(lchan));
+			LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Got ciphering conf with empty queue\n");
 			goto err;
 		}
 
@@ -1430,10 +1416,8 @@ static int mph_send_config_logchpar(struct gsm_lchan *lchan, struct sapi_cmd *cm
 
 	/* FIXME: update encryption */
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-CONFIG.req (%s) ",
-		gsm_lchan_name(lchan),
-		get_value_string(oc2gbts_l1sapi_names,
-				 conf_req->cfgParams.setLogChParams.sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-CONFIG.req (%s) ",
+		  get_value_string(oc2gbts_l1sapi_names, conf_req->cfgParams.setLogChParams.sapi));
 	LOGPC(DL1C, LOGL_INFO, "cfgParams Tn=%u, subCh=%u, dir=0x%x ",
 			conf_req->cfgParams.setLogChParams.u8Tn,
 			conf_req->cfgParams.setLogChParams.subCh,
@@ -1541,11 +1525,9 @@ static int mph_send_config_ciphering(struct gsm_lchan *lchan, struct sapi_cmd *c
 		return -EINVAL;
 	cfgr->cfgParams.setCipheringParams.cipherId = rsl2l1_ciph[lchan->encr.alg_id];
 
-	LOGP(DL1C, LOGL_NOTICE, "%s SET_CIPHERING (ALG=%u %s)\n",
-		gsm_lchan_name(lchan),
-		cfgr->cfgParams.setCipheringParams.cipherId,
-		get_value_string(oc2gbts_dir_names,
-				 cfgr->cfgParams.setCipheringParams.dir));
+	LOGPLCHAN(lchan, DL1C, LOGL_NOTICE, "SET_CIPHERING (ALG=%u %s)\n",
+		   cfgr->cfgParams.setCipheringParams.cipherId,
+		   get_value_string(oc2gbts_dir_names, cfgr->cfgParams.setCipheringParams.dir));
 
 	memcpy(cfgr->cfgParams.setCipheringParams.u8Kc,
 	       lchan->encr.key, lchan->encr.key_len);
@@ -1623,9 +1605,8 @@ static int lchan_deact_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 		goto err;
 	}
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-DEACTIVATE.conf (%s ",
-		gsm_lchan_name(lchan),
-		get_value_string(oc2gbts_l1sapi_names, ic->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-DEACTIVATE.conf (%s ",
+		  get_value_string(oc2gbts_l1sapi_names, ic->sapi));
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(oc2gbts_dir_names, ic->dir));
 
@@ -1647,19 +1628,15 @@ static int lchan_deact_compl_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 
 
 	if (llist_empty(&lchan->sapi_cmds)) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Got de-activation confirmation with empty queue\n",
-				gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Got de-activation confirmation with empty queue\n");
 		goto err;
 	}
 
 	cmd = llist_entry(lchan->sapi_cmds.next, struct sapi_cmd, entry);
 	if (cmd->sapi != ic->sapi || cmd->dir != ic->dir ||
 			cmd->type != SAPI_CMD_DEACTIVATE) {
-		LOGP(DL1C, LOGL_ERROR,
-				"%s Confirmation mismatch (%d, %d) (%d, %d)\n",
-				gsm_lchan_name(lchan), cmd->sapi, cmd->dir,
-				ic->sapi, ic->dir);
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "Confirmation mismatch (%d, %d) (%d, %d)\n",
+			  cmd->sapi, cmd->dir, ic->sapi, ic->dir);
 		goto err;
 	}
 
@@ -1684,9 +1661,8 @@ static int mph_send_deactivate_req(struct gsm_lchan *lchan, struct sapi_cmd *cmd
 	deact_req->sapi = cmd->sapi;
 	deact_req->hLayer3 = (HANDLE)l1if_lchan_to_hLayer(lchan);
 
-	LOGP(DL1C, LOGL_INFO, "%s MPH-DEACTIVATE.req (%s ",
-		gsm_lchan_name(lchan),
-		get_value_string(oc2gbts_l1sapi_names, deact_req->sapi));
+	LOGPLCHAN(lchan, DL1C, LOGL_INFO, "MPH-DEACTIVATE.req (%s ",
+		  get_value_string(oc2gbts_l1sapi_names, deact_req->sapi));
 	LOGPC(DL1C, LOGL_INFO, "%s)\n",
 		get_value_string(oc2gbts_dir_names, deact_req->dir));
 
@@ -1698,8 +1674,7 @@ static int sapi_deactivate_cb(struct gsm_lchan *lchan, int status)
 {
 	/* FIXME: Error handling. There is no NACK... */
 	if (status != GsmL1_Status_Success && lchan->state == LCHAN_S_REL_REQ) {
-		LOGP(DL1C, LOGL_ERROR, "%s is now broken. Stopping the release.\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "is now broken. Stopping the release.\n");
 		lchan_set_state(lchan, LCHAN_S_BROKEN);
 		sapi_clear_queue(&lchan->sapi_cmds);
 		mph_info_chan_confirm(lchan, PRIM_INFO_DEACTIVATE, 0);
@@ -1784,8 +1759,7 @@ static int lchan_deactivate_sapis(struct gsm_lchan *lchan)
 
 	/* nothing was queued */
 	if (res == 0) {
-		LOGP(DL1C, LOGL_ERROR, "%s all SAPIs already released?\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "all SAPIs already released?\n");
 		lchan_set_state(lchan, LCHAN_S_BROKEN);
 		mph_info_chan_confirm(lchan, PRIM_INFO_DEACTIVATE, 0);
 	}
@@ -1836,28 +1810,14 @@ int bts_model_check_oml(struct gsm_bts *bts, uint8_t msg_type,
 }
 
 /* callback from OML */
-int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
-			struct tlv_parsed *new_attr, int kind, void *obj)
+int bts_model_apply_oml(struct gsm_bts *bts, const struct msgb *msg,
+			struct gsm_abis_mo *mo, void *obj)
 {
 	struct abis_om_fom_hdr *foh = msgb_l3(msg);
-	struct gsm_abis_mo *mo = gsm_objclass2mo(bts, foh->obj_class, &foh->obj_inst);
-	struct nm_fsm_ev_setattr_data ev_data = {
-		.msg = msg,
-		.cause = 0,
-	};
-	int rc;
 	struct gsm_bts_trx *trx;
 	struct oc2gl1_hdl *fl1h;
 	uint8_t cell_size;
 
-	/* TODO: NM Object without FSM: */
-	switch (foh->obj_class) {
-	case NM_OC_GPRS_NSE:
-	case NM_OC_GPRS_CELL:
-	case NM_OC_GPRS_NSVC:
-		return oml_fom_ack_nack(ev_data.msg, ev_data.cause);
-	}
-
 	switch (foh->msg_type) {
 	case NM_MT_SET_RADIO_ATTR:
 		trx = obj;
@@ -1895,11 +1855,7 @@ int bts_model_apply_oml(struct gsm_bts *bts, struct msgb *msg,
 		break;
 	}
 
-	rc = osmo_fsm_inst_dispatch(mo->fi,
-				    ev_data.cause == 0 ? NM_EV_SETATTR_ACK : NM_EV_SETATTR_NACK,
-				    &ev_data);
-	/* msgb ownsership is transferred to FSM if it received ev: */
-	return rc == 0 ? 1 : 0;
+	return 0;
 }
 
 /* callback from OML */
@@ -1913,32 +1869,20 @@ int bts_model_opstart(struct gsm_bts *bts, struct gsm_abis_mo *mo,
 
 	switch (mo->obj_class) {
 	case NM_OC_SITE_MANAGER:
-		rc = osmo_fsm_inst_dispatch(bts->site_mgr.mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
 	case NM_OC_BTS:
-		rc = osmo_fsm_inst_dispatch(bts->mo.fi, NM_EV_OPSTART_ACK, NULL);
-		oml_mo_state_chg(&bts->mo, -1, NM_AVSTATE_OK, -1);
-		oml_mo_state_chg(&bts->gprs.nse.mo, -1, NM_AVSTATE_OK, -1);
-		oml_mo_state_chg(&bts->gprs.cell.mo, -1, NM_AVSTATE_OK, -1);
-		oml_mo_state_chg(&bts->gprs.nsvc[0].mo, -1, NM_AVSTATE_OK, -1);
+	case NM_OC_BASEB_TRANSC:
+	case NM_OC_GPRS_NSE:
+	case NM_OC_GPRS_CELL:
+	case NM_OC_GPRS_NSVC:
+		rc = osmo_fsm_inst_dispatch(mo->fi, NM_EV_OPSTART_ACK, NULL);
 		break;
 	case NM_OC_RADIO_CARRIER:
 		trx = (struct gsm_bts_trx *) obj;
 		rc = trx_init(trx);
 		break;
-	case NM_OC_BASEB_TRANSC:
-		bb_transc = (struct gsm_bts_bb_trx *) obj;
-		rc = osmo_fsm_inst_dispatch(bb_transc->mo.fi, NM_EV_OPSTART_ACK, NULL);
-		break;
 	case NM_OC_CHANNEL:
 		ts = (struct gsm_bts_trx_ts*) obj;
-		rc = ts_opstart(obj);
-		break;
-	case NM_OC_GPRS_NSE:
-	case NM_OC_GPRS_CELL:
-	case NM_OC_GPRS_NSVC:
-		oml_mo_state_chg(mo, NM_OPSTATE_ENABLED, -1, -1);
-		rc = oml_mo_opstart_ack(mo);
+		rc = ts_opstart(ts);
 		break;
 	default:
 		rc = oml_mo_opstart_nack(mo, NM_NACK_OBJCLASS_NOTSUPP);
@@ -2009,14 +1953,10 @@ int l1if_rsl_chan_act(struct gsm_lchan *lchan)
  */
 int l1if_rsl_chan_mod(struct gsm_lchan *lchan)
 {
-	const struct lchan_sapis *s4l = &sapis_for_lchan[lchan->type];
-	unsigned int i;
-
 	if (lchan->ho.active == HANDOVER_NONE)
 		return -1;
 
-	LOGP(DHO, LOGL_ERROR, "%s modifying channel for handover\n",
-		gsm_lchan_name(lchan));
+	LOGPLCHAN(lchan, DHO, LOGL_ERROR, "modifying channel for handover\n");
 
 	/* Give up listening to RACH bursts */
 	release_sapi_ul_rach(lchan);
@@ -2032,8 +1972,7 @@ int l1if_rsl_chan_rel(struct gsm_lchan *lchan)
 {
 	/* A duplicate RF Release Request, ignore it */
 	if (lchan->state == LCHAN_S_REL_REQ) {
-		LOGP(DL1C, LOGL_ERROR, "%s already in release request state.\n",
-			gsm_lchan_name(lchan));
+		LOGPLCHAN(lchan, DL1C, LOGL_ERROR, "already in release request state.\n");
 		return 0;
 	}
 
@@ -2069,8 +2008,7 @@ static int ts_disconnect_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 	struct gsm_bts_trx_ts *ts = &trx->ts[cnf->u8Tn];
 	OSMO_ASSERT(cnf->u8Tn < TRX_NR_TS);
 
-	LOGP(DL1C, LOGL_DEBUG, "%s Rx mphDisconnectCnf\n",
-	     gsm_lchan_name(ts->lchan));
+	LOGPLCHAN(ts->lchan, DL1C, LOGL_DEBUG, "Rx mphDisconnectCnf\n");
 
 	cb_ts_disconnected(ts);
 
@@ -2085,7 +2023,7 @@ int bts_model_ts_disconnect(struct gsm_bts_trx_ts *ts)
 	struct oc2gl1_hdl *fl1h = trx_oc2gl1_hdl(ts->trx);
 	GsmL1_MphDisconnectReq_t *cr;
 
-	DEBUGP(DRSL, "%s TS disconnect\n", gsm_lchan_name(ts->lchan));
+	LOGPLCHAN(ts->lchan, DRSL, LOGL_DEBUG, "TS disconnect\n");
 	cr = prim_init(msgb_l1prim(msg), GsmL1_PrimId_MphDisconnectReq, fl1h,
 		       l1p_handle_for_ts(ts));
 	cr->u8Tn = ts->nr;
@@ -2101,12 +2039,11 @@ static int ts_connect_cb(struct gsm_bts_trx *trx, struct msgb *l1_msg,
 	struct gsm_bts_trx_ts *ts = &trx->ts[cnf->u8Tn];
 	OSMO_ASSERT(cnf->u8Tn < TRX_NR_TS);
 
-	DEBUGP(DL1C, "%s %s Rx mphConnectCnf flags=%s%s%s\n",
-	       gsm_lchan_name(ts->lchan),
-	       gsm_pchan_name(ts->pchan),
-	       ts->flags & TS_F_PDCH_ACTIVE ? "ACTIVE " : "",
-	       ts->flags & TS_F_PDCH_ACT_PENDING ? "ACT_PENDING " : "",
-	       ts->flags & TS_F_PDCH_DEACT_PENDING ? "DEACT_PENDING " : "");
+	LOGPLCHAN(ts->lchan, DL1C, LOGL_DEBUG, "%s Rx mphConnectCnf flags=%s%s%s\n",
+		  gsm_pchan_name(ts->pchan),
+		  ts->flags & TS_F_PDCH_ACTIVE ? "ACTIVE " : "",
+		  ts->flags & TS_F_PDCH_ACT_PENDING ? "ACT_PENDING " : "",
+		  ts->flags & TS_F_PDCH_DEACT_PENDING ? "DEACT_PENDING " : "");
 
 	cb_ts_connected(ts, 0);
 
