@@ -1,5 +1,5 @@
 /*
- * (C) 2020-2021 by sysmocom - s.m.f.c. GmbH <info@sysmocom.de>
+ * (C) 2020-2021 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Vadim Yanitskiy <vyanitskiy@sysmocom.de>
  *
  * All Rights Reserved
@@ -12,7 +12,7 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -47,14 +47,8 @@
 	{ DL_MEAS_FULL(rxqual, rxlev), \
 	  DL_MEAS_SUB(rxqual, rxlev) }
 
-#define DL_MEAS_FULL_SUB_INV(rxqual, rxlev) \
-	{ DL_MEAS_FULL(rxqual, rxlev), \
-	  DL_MEAS_SUB(rxqual, rxlev), \
-	  .invalid = true }
-
 enum power_test_step_type {
 	PWR_TEST_ST_IND_MEAS = 0,
-	PWR_TEST_ST_IND_DUMMY,
 	PWR_TEST_ST_SET_STATE,
 	PWR_TEST_ST_SET_CTRL_INTERVAL,
 	PWR_TEST_ST_SET_STEP_SIZE,
@@ -78,7 +72,6 @@ struct power_test_step {
 			uint8_t rxqual_sub;
 			uint8_t rxlev_full;
 			uint8_t rxlev_sub;
-			bool invalid;
 		} meas;
 		/* Increase / reduce step size */
 		struct {
@@ -112,31 +105,32 @@ static void init_test(const char *name)
 	g_bts->band = GSM_BAND_900;
 	g_bts->c0 = g_trx;
 
+	/* Init defaultBS power control parameters, enable dynamic power control */
+	struct gsm_power_ctrl_params *params = &g_trx->ts[0].lchan[0].bs_dpc_params;
+	g_trx->ts[0].lchan[0].bs_power_ctrl.dpc_params = params;
+	*params = power_ctrl_params_def;
+
+	/* Disable loop SACCH block skip by default: */
+	params->ctrl_interval = 0;
+
 	printf("\nStarting test case '%s'\n", name);
 }
 
-static void enc_meas_rep(struct gsm48_hdr *gh,
+static void enc_meas_rep(struct gsm48_meas_res *mr,
 			 const unsigned int n,
 			 const struct power_test_step *step)
 {
-	struct gsm48_meas_res *mr = (struct gsm48_meas_res *) gh->data;
-
-	gh->proto_discr = GSM48_PDISC_RR;
-	gh->msg_type = GSM48_MT_RR_MEAS_REP;
-
 	*mr = (struct gsm48_meas_res) {
 		.rxlev_full = step->meas.rxlev_full,
 		.rxlev_sub = step->meas.rxlev_sub,
 		.rxqual_full = step->meas.rxqual_full,
 		.rxqual_sub = step->meas.rxqual_sub,
-		/* NOTE: inversed logic (1 means invalid) */
-		.meas_valid = step->meas.invalid,
 	};
 
-	printf("#%02u %s() -> Measurement Results (%svalid): "
+	printf("#%02u %s() -> Measurement Results (valid): "
 	       "RXLEV-FULL(%02u), RXQUAL-FULL(%u), "
 	       "RXLEV-SUB(%02u), RXQUAL-SUB(%u)\n",
-	       n, __func__, step->meas.invalid ? "in" : "",
+	       n, __func__,
 	       mr->rxlev_full, mr->rxqual_full,
 	       mr->rxlev_sub, mr->rxqual_sub);
 }
@@ -145,11 +139,8 @@ static int exec_power_step(struct gsm_lchan *lchan,
 			   const unsigned int n,
 			   const struct power_test_step *step)
 {
-	struct gsm48_hdr *gh;
+	struct gsm48_meas_res mr;
 	uint8_t old, new;
-	uint8_t buf[18];
-
-	gh = (struct gsm48_hdr *) buf;
 
 	switch (step->type) {
 	case PWR_TEST_ST_SET_STATE:
@@ -184,20 +175,16 @@ static int exec_power_step(struct gsm_lchan *lchan,
 		printf("#%02u %s() <- Enable DTXd\n", n, __func__);
 		lchan->tch.dtx.dl_active = true;
 		return 0; /* we're done */
-	case PWR_TEST_ST_IND_DUMMY:
-		printf("#%02u %s() <- Dummy block\n", n, __func__);
-		memset(buf, 0x2b, sizeof(buf));
-		break;
 	case PWR_TEST_ST_IND_MEAS:
-		enc_meas_rep(gh, n, step);
+		enc_meas_rep(&mr, n, step);
 		break;
 	}
 
-	printf("#%02u lchan_bs_pwr_ctrl() <- UL SACCH: %s\n",
-	       n, osmo_hexdump(buf, sizeof(buf)));
+	printf("#%02u lchan_bs_pwr_ctrl() <- UL SACCH: 06 15 %s\n",
+	       n, osmo_hexdump((void *)&mr, sizeof(mr)));
 
 	old = lchan->bs_power_ctrl.current;
-	lchan_bs_pwr_ctrl(lchan, gh);
+	lchan_bs_pwr_ctrl(lchan, &mr);
 	new = lchan->bs_power_ctrl.current;
 
 	printf("#%02u lchan_bs_pwr_ctrl() -> BS power reduction: "
@@ -219,9 +206,6 @@ static void exec_power_test(const struct power_test_step *steps,
 	struct gsm_lchan *lchan = &g_trx->ts[0].lchan[0];
 	struct gsm_power_ctrl_params *params = &lchan->bs_dpc_params;
 
-	/* Default BS power control parameters */
-	memcpy(params, &power_ctrl_params_def, sizeof(*params));
-
 	/* No RxLev hysteresis: lower == upper */
 	params->rxlev_meas.lower_thresh = PWR_TEST_RXLEV_TARGET;
 	params->rxlev_meas.upper_thresh = PWR_TEST_RXLEV_TARGET;
@@ -401,24 +385,6 @@ static const struct power_test_step TC_rxqual_ber[] = {
 	{ .meas = DL_MEAS_FULL_SUB(7, PWR_TEST_RXLEV_TARGET) }, /* max */
 };
 
-/* Verify that invalid and dummy SACCH blocks are ignored. */
-static const struct power_test_step TC_inval_dummy[] = {
-	/* Initial state: 16 dB, up to 20 dB */
-	{ .type = PWR_TEST_ST_SET_STATE,
-	  .state = { .current = 16, .max = 2 * 10 } },
-
-	/* MS sends invalid measurement results which must be ignored */
-	{ .meas = DL_MEAS_FULL_SUB_INV(7, 63),			.exp_txred = 16 },
-	{ .meas = DL_MEAS_FULL_SUB_INV(0, 0),			.exp_txred = 16 },
-
-	/* Let's say SMS (SAPI=3) blocks substitute some of the reports */
-	{ .meas = DL_MEAS_FULL_SUB(0, PWR_TEST_RXLEV_TARGET),	.exp_txred = 16 },
-	{ .type = PWR_TEST_ST_IND_DUMMY, /* not a report */	.exp_txred = 16 },
-	{ .meas = DL_MEAS_FULL_SUB(0, PWR_TEST_RXLEV_TARGET),	.exp_txred = 16 },
-	{ .type = PWR_TEST_ST_IND_DUMMY, /* not a report */	.exp_txred = 16 },
-	{ .meas = DL_MEAS_FULL_SUB(0, PWR_TEST_RXLEV_TARGET),	.exp_txred = 16 },
-};
-
 /* Verify handling of optional power control interval (P_Con_INTERVAL). */
 static const struct power_test_step TC_ctrl_interval[] = {
 	/* Initial state: 0 dB, up to 20 dB */
@@ -540,7 +506,6 @@ int main(int argc, char **argv)
 
 	exec_test(TC_dtxd_mode);
 	exec_test(TC_rxqual_ber);
-	exec_test(TC_inval_dummy);
 	exec_test(TC_ctrl_interval);
 
 	exec_test(TC_rxlev_hyst);
